import sys
import boto3
import botocore
import click

client = boto3.client('cloudformation')


def validate_template(client, filename):
    try:
        with open(filename, 'r') as f:
            template = f.read()
            client.validate_template(TemplateBody=template)
    except botocore.exceptions.ClientError as e:
        click.echo(e.response["Error"]["Message"])
        sys.exit(1)


@click.command(short_help='Validate template')
@click.option('-p', '--profile', default=None, help='AWS profile')
@click.option('-r', '--region', default=None, help='AWS region')
@click.argument('filename')
def validate(filename, profile, region):
    """Validate template

    \b
    cfn-tools validate TEMPLATE
    """
    session = boto3.session.Session(profile_name=profile, region_name=region)
    client = session.client('cloudformation')

    validate_template(client, filename)


if __name__ == '__main__':
    validate()



import sys
import boto3
import click

from botocore.exceptions import ClientError


status_map = {
    'CREATE_IN_PROGRESS': 'CP',
    'CREATE_FAILED': 'CF',
    'CREATE_COMPLETE': 'C',
    'ROLLBACK_IN_PROGRESS': 'RP',
    'ROLLBACK_FAILED': 'RF',
    'ROLLBACK_COMPLETE': 'R',
    'DELETE_IN_PROGRESS': 'DP',
    'DELETE_FAILED': 'DF',
    'DELETE_COMPLETE': 'D',
    'UPDATE_IN_PROGRESS': 'UP',
    'UPDATE_COMPLETE_CLEANUP_IN_PROGRESS': 'UCP',
    'UPDATE_COMPLETE': 'U',
    'UPDATE_ROLLBACK_IN_PROGRESS': 'URP',
    'UPDATE_ROLLBACK_FAILED': 'URF',
    'UPDATE_ROLLBACK_COMPLETE_CLEANUP_IN_PROGRESS': 'URCP',
    'UPDATE_ROLLBACK_COMPLETE': 'UR',
    'REVIEW_IN_PROGRESS': 'RevP'
}


def get_stacks(client, filters):
    """Returns a list of StackSummaries"""
    try:
        response = client.list_stacks(StackStatusFilter=filters)
        stacks = response['StackSummaries']
        while True:
            if 'NextToken' not in response:
                break
            response = client.list_stacks(
                NextToken=response['NextToken'], StackStatusFilter=filters)
            stacks += response['StackSummaries']
        return stacks
    except ClientError as e:
        click.echo(e.response["Error"]["Message"])
        sys.exit(1)


def get_filters(filter):
    """Returns a list of filtered stack status codes"""
    if filter is None or 'COMPLETE' in filter.upper():
        return [k for k in status_map.keys() if 'DELETE_' not in k]
    elif filter.startswith('!') is True:
        return [k for k in status_map.keys() if filter[1:].upper() not in k and 'DELETE_' not in k]
    else:
        return [k for k in status_map.keys() if filter.upper() in k]


def list_codes(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    for k, v in sorted(status_map.items()):
        click.echo('{0:>4} {1}'.format(v, k))
    ctx.exit()


def format_listing(stack, time_label):
    s = stack
    click.echo('{0:<18} {1:<4} {2}'.format(
        s[time_label].strftime("%Y-%m-%d %H:%M"), status_map[s['StackStatus']], s['StackName']))


@click.command(short_help='List stacks')
@click.option('-f', '--filter', help='filter stacks by status code')
@click.option('-c', '--codes', is_flag=True, callback=list_codes, expose_value=False,
              is_eager=True, help='List status codes')
@click.option('-p', '--profile', default=None, help='AWS profile')
@click.option('-r', '--region', default=None, help='AWS region')
@click.argument('name', default='')
def ls(name, filter, profile, region):
    """List stacks

    Deleted stacks are filtered out by default

    \b
    cfn-tools ls
    cfn-tools ls FUZZY_SEARCH

    \b
    cfn-tools --codes
    cfn-tools ls FUZZY_SEARCH -f update
    cfn-tools ls FUZZY_SEARCH -f \!update
    """

    session = boto3.session.Session(profile_name=profile, region_name=region)
    client = session.client('cloudformation')

    filters = get_filters(filter)
    stacks = get_stacks(client, filters)
    if name is not None:
        stacks = [k for k in stacks if name in k['StackName']]

    for s in stacks:
        if 'LastUpdatedTime' in s:
            format_listing(s, 'LastUpdatedTime')
        elif 'DeletionTime' in s:
            format_listing(s, 'DeletionTime')
        else:
            format_listing(s, 'CreationTime')



import sys
import boto3
import botocore
import click
import json
import difflib

from cfntools.validate import validate_template


def get_template(client, stack):
    """Get the template for a stack"""
    try:
        response = client.get_template(
            StackName=stack
        )
        template = response["TemplateBody"]
        if isinstance(template, dict):
            template = json.dumps(template, indent=2, sort_keys=True)
        return template
    except botocore.exceptions.ClientError as e:
        click.echo(e.response["Error"]["Message"])
        sys.exit(1)


def read_file(file):
    try:
        with open(file, 'r') as f:
            return f.read()
    except IOError as e:
        click.echo(e)
        sys.exit(1)


def read_template(client, template):
    """Read the template from a file or the stack object"""
    if template.endswith(tuple(['.yaml', '.yml', '.json'])):
        validate_template(client, template)
        tmpl = read_file(template)
        if template.endswith('.json'):
            tmpl = json.dumps(json.loads(tmpl), indent=2, sort_keys=True)
    else:  # CloudFormation Stack
        tmpl = get_template(client, template)
    return tmpl


def create_cloudformation_session(profile=None, region=None):
    session = boto3.session.Session(profile_name=profile, region_name=region)
    return session.client('cloudformation')


@click.command(short_help='Diff stacks and/or templates')
@click.option('-p', '--profile', help='AWS profile')
@click.option('-r', '--region', help='AWS region')
@click.option('-p2', '--profile2', help='AWS profile (optional for second template)')
@click.option('-r2', '--region2', help='AWS region (optional for second template)')
@click.argument('TEMPLATE1')
@click.argument('TEMPLATE2')
def diff(template1, template2, profile, region, profile2, region2):
    """Diff stacks and/or templates

    \b
    cfn-tools diff TEMPLATE STACK
    cfn-tools diff STACK1 STACK2
    cfn-tools diff -r us-west-2 STACK1 -r2 us-east-1 STACK2
    """
    region2 = region2 or region
    profile2 = profile2 or profile

    client1 = create_cloudformation_session(profile, region)
    client2 = create_cloudformation_session(profile2, region2)

    tmpl1 = read_template(client1, template1)
    tmpl2 = read_template(client2, template2)

    diff = difflib.unified_diff(
        tmpl1.splitlines(),
        tmpl2.splitlines(),
        fromfile='a/' + template1,
        tofile='b/' + template2
    )
    for line in diff:
        if line.startswith("+"):
            click.secho(line, fg='green')
        elif line.startswith("-"):
            click.secho(line, fg='red')
        else:
            click.echo(line)


if __name__ == '__main__':
    diff()



# -*- coding: utf-8 -*-

import click

from cfntools.ls import ls
from cfntools.diff import diff
from cfntools.validate import validate

version = '0.1.6'


@click.group()
@click.version_option(version=version, message=version)
def main(args=None):
    """Tools for AWS CloudFormation

       \b
       cfn-tools --help
       cfn-tools <command> --help
    """


main.add_command(ls)
main.add_command(diff)
main.add_command(validate)


if __name__ == "__main__":
    main()



import setuptools

requirements = [
    'Click>=6.0',
    'boto3>=1.3.1'
]

setuptools.setup(
    name="cfn-tools",
    version="0.1.6",
    url="https://github.com/boroivanov/cfn-tools",

    author="Borislav Ivanov",
    author_email="borogl@gmail.com",

    description="Tools for AWS CloudFormation",
    long_description=open('README.rst').read(),

    packages=[
        'cfntools',
    ],
    package_dir={'cfntools':
                 'cfntools'},
    entry_points={
        'console_scripts': [
            'cfn-tools=cfntools.cli:main'
        ]
    },

    install_requires=requirements,
    license="MIT license",

    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
    ],
)
